@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 11270 2018-11-29 01:56:59Z Kevin $
+# $Id: __init__.py 11289 2018-12-01 18:42:04Z Lavender $
 #
 # Copyright (c) 2015 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -11,9 +11,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# $Author: Kevin $ (last)
-# $Date: 2018-11-29 10:56:59 +0900 (週四, 29 十一月 2018) $
-# $Revision: 11270 $
+# $Author: Lavender $ (last)
+# $Date: 2018-12-02 03:42:04 +0900 (週日, 02 十二月 2018) $
+# $Revision: 11289 $
 
 from __future__ import print_function
 
@@ -872,6 +872,22 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
 
     #=========================================================================
 
+    #=========================================================================
+    # Deploy configure command
+    EXTRA_SETTINGS = {
+        'IUNO_INITIAL_MEDIA': 'initial_data.zip',
+    }
+    if not 'IUNO_INITIAL_MEDIA' in settingsDict:
+        update(settingsDict, EXTRA_SETTINGS)
+
+    if 'IUNO_SITE_DOMAIN_NAMES' in settingsDict:
+        siteNames = settingsDict['IUNO_SITE_DOMAIN_NAMES']
+        for mode in [DEVELOPMENT, STAGE, PRODUCTION]:
+            if not mode in siteNames:
+                raise KeyError(
+                    "You should set DEVELOPMENT, STAGE, "
+                    "PRODUCTION in IUNO_SITE_DOMAIN_NAMES")
+
 
 # detect Server mode
 def detectServerMode(settingsDict):
