@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 11277 2018-11-29 07:20:01Z Lavender $
+# $Id: __init__.py 11306 2018-12-12 15:16:07Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $ (last)
-# $Date: 2018-11-29 16:20:01 +0900 (週四, 29 十一月 2018) $
-# $Revision: 11277 $
+# $Date: 2018-12-13 00:16:07 +0900 (週四, 13 十二月 2018) $
+# $Revision: 11306 $
 
 import os
 import sys
@@ -365,54 +365,69 @@ def attachSettings(settingsDict, enablePaypal=False):
 
     #=========================================================================
     # Django order status settings
+    allSettings = [
+        'OSCAR_INITIAL_ORDER_STATUS',
+        'OSCAR_INITIAL_LINE_STATUS',
+        'OSCAR_ORDER_STATUS_PIPELINE',
+        'OSCAR_ORDER_SEND_EMAIL_STATUS',
+
+        'OSCAR_ORDER_RETURN_STATUS',
+        'OSCAR_ORDER_PROCESSING_STATUS',
+        'OSCAR_ORDER_COMPLETE_STATUS',
+    ]
+
+    # add default settings
     needShipping = False
     if 'IUNO_SHOP_NEED_SHIPPING' in settingsDict:
         needShipping = settingsDict['IUNO_SHOP_NEED_SHIPPING']
-    """
-    ADDING_OSCAR_ORDER_STATUS = {
-        'OSCAR_INITIAL_ORDER_STATUS': _('Order pending'),
-        'OSCAR_INITIAL_LINE_STATUS': '-',
-        'OSCAR_ORDER_STATUS_PIPELINE': {
-            _('Order pending'): \
-                (_('Order being processed'), _('Order cancelled'),),
-            _('Order being processed'): \
-                (_('Order complete'), _('Order cancelled'),),
-            _('Order cancelled'): (),
-            _('Order complete'): (),
-            _('Order return'): (_('Order return complete'),),
-            _('Order return complete'): (),
-        }
-    }
-    """
-    if needShipping:
-        ADDING_OSCAR_ORDER_STATUS = {
-            'OSCAR_INITIAL_ORDER_STATUS': u'等待處理中',
-            'OSCAR_INITIAL_LINE_STATUS': '-',
-            'OSCAR_ORDER_STATUS_PIPELINE': {
-                u'等待處理中': \
-                    (u'商品送貨中', u'訂單取消', u'測試用訂單'),
-                u'商品送貨中': \
-                    (u'完成訂單', u'訂單取消', u'測試用訂單'),
-                u'訂單取消': (u'測試用訂單', ),
-                u'完成訂單': (u'測試用訂單', ),
+    
+    hasConfig = False
+    for s in allSettings:
+        if s in settingsDict:
+             hasConfig = True
+             break
 
-                u'商品退貨中': (u'商品退貨中',),
-                u'測試用訂單': (u'等待處理中',),
+    if not hasConfig:
+        if needShipping:
+            ADDING_OSCAR_ORDER_STATUS = {
+                'OSCAR_ORDER_SEND_EMAIL_STATUS': u'商品送貨中',
+                'OSCAR_INITIAL_ORDER_STATUS': u'等待處理中',
+                'OSCAR_INITIAL_LINE_STATUS': '-',
+                'OSCAR_ORDER_RETURN_STATUS': u'商品退貨中',
+                'OSCAR_ORDER_PROCESSING_STATUS': u'商品送貨中',
+                'OSCAR_ORDER_COMPLETE_STATUS': u'完成訂單',
+                'OSCAR_ORDER_STATUS_PIPELINE': {
+                    u'等待處理中': \
+                        (u'商品送貨中', u'訂單取消', u'測試用訂單'),
+                    u'商品送貨中': \
+                        (u'完成訂單', u'訂單取消', u'測試用訂單'),
+                    u'訂單取消': (u'測試用訂單', ),
+                    u'完成訂單': (u'測試用訂單', ),
+
+                    u'商品退貨中': (u'完成退貨', u'測試用訂單'),
+                    u'完成退貨': (u'測試用訂單',),
+                    u'測試用訂單': (u'等待處理中',),
+                }
             }
-        }
+        else:
+            ADDING_OSCAR_ORDER_STATUS = {
+                'OSCAR_INITIAL_ORDER_STATUS': u'完成訂單',
+                'OSCAR_INITIAL_LINE_STATUS': '-',
+                'OSCAR_ORDER_STATUS_PIPELINE': {
+                    u'完成訂單': (u'測試用訂單', ),
+                    u'測試用訂單': (u'完成訂單',),
+                }
+            }
+
+        update(settingsDict, ADDING_OSCAR_ORDER_STATUS)
     else:
-        ADDING_OSCAR_ORDER_STATUS = {
-            'OSCAR_INITIAL_ORDER_STATUS': u'完成訂單',
-            'OSCAR_INITIAL_LINE_STATUS': '-',
-            'OSCAR_ORDER_STATUS_PIPELINE': {
-                u'完成訂單': (u'測試用訂單', ),
-                u'測試用訂單': (u'完成訂單',),
-            }
-        }
-
-    update(settingsDict, ADDING_OSCAR_ORDER_STATUS)
+        # check settings:
+        unsettings = [s for s in allSettings if not s in settingsDict]
+        if unsettings:
+            raise KeyError("You should also set '%s' value." % str(unsettings))
     
     #=========================================================================
+
     
 
 def attachShopURLs(settings, urlpatterns):
@@ -568,7 +583,7 @@ class AllPay2(AllPay):
             form_html += utf8("<script>document.allPayForm.submit();</script>")
         return form_html
 
-def buildAPIForm(submission, request, apiName):
+def buildAPIForm(submission, request, apiName, extra={}):
     scheme = request.scheme
     domain = request.META['HTTP_HOST']
 
@@ -588,11 +603,6 @@ def buildAPIForm(submission, request, apiName):
     for index, line in enumerate(basket.all_lines()):
         product = line.product
         
-        # this is for megaoil
-        if hasattr(settings, 'PRODUCT_UPC_PRICE_ADJUST'):
-            if product.upc in settings.PRODUCT_UPC_PRICE_ADJUST:
-                total = total + settings.PRODUCT_UPC_PRICE_ADJUST[product.upc]
-        
         productName = product.get_title()
         itemName += utf8(productName)
         
@@ -739,6 +749,7 @@ def buildAPIForm(submission, request, apiName):
             'TradeDesc' : settings.OSCAR_SHOP_NAME, 
             'Remark' : settings.OSCAR_SHOP_NAME,
         }
+
         if submission['shipping_address']:
             needExtraData = False
         else:
@@ -757,17 +768,9 @@ def buildAPIForm(submission, request, apiName):
                                       
         return form
 
-    if apiName == ECPAY_LOGISTIC and request.POST:
-        storeId = request.POST.get("CVSStoreID")
-        subType = request.POST.get("LogisticsSubType")
-        extra = request.POST.get("ExtraData")
-
-        # 防呆 or submission 內會有地址資料
-        print submission['shipping_address']
-        splitData = extra.split(",")
-        name = splitData[1][len("name="):]
-        phone = splitData[0][len("phone="):]
-
+    if apiName == ECPAY_LOGISTIC and request.session.get('logisticMap'):
+        mapData = json.loads(request.session['logisticMap'])
+        
         if settings.ECPAY_SANDBOX_MODE:
             apiUrl = "https://logistics-stage.ecpay.com.tw/Express/Create"
         else:
@@ -783,6 +786,11 @@ def buildAPIForm(submission, request, apiName):
         else:
             shopPhone = ''
 
+        if "isCollection" in extra:
+            isCollection = extra["isCollection"]
+        else:
+            isCollection = "Y"
+
         data = {
             "MerchantID": settings.ECPAY_API_MERCHANT_ID,
             "MerchantTradeNo": merchantTradeNo,
@@ -790,9 +798,9 @@ def buildAPIForm(submission, request, apiName):
                                     "%Y/%m/%d %H:%M:%S"),
 
             "LogisticsType": "CVS",
-            "LogisticsSubType": subType,
-            "IsCollection": "Y",
-            "ReceiverStoreID": storeId,
+            "LogisticsSubType": mapData['storeType'],
+            "IsCollection": isCollection,
+            "ReceiverStoreID": mapData['storeId'],
             
             "GoodsAmount": int(total),
             "CollectionAmount": int(total),
@@ -801,10 +809,12 @@ def buildAPIForm(submission, request, apiName):
             # 必須 len(SendName) <= 10 個字元 
             "SenderName": shopName,
             "SenderCellPhone": shopPhone,
-            "ReceiverName": name, 
-            "ReceiverCellPhone": phone, 
+
+            "ReceiverName": mapData['receiverName'], 
+            "ReceiverCellPhone": mapData['receiverPhone'], 
             
-            "ServerReplyURL": reverse("ecpayLogisticReply"), # 隨時接訂單資訊的 URL
+            # 隨時接訂單資訊的 URL
+            "ServerReplyURL": reverse("ecpayLogisticReply"), 
             "ClientReplyURL": returnUrl,               
         }
         
