@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: views.py 11277 2018-11-29 07:20:01Z Lavender $
+# $Id: views.py 11295 2018-12-05 13:49:49Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $
-# $Date: 2018-11-29 16:20:01 +0900 (週四, 29 十一月 2018) $
-# $Revision: 11277 $ 
+# $Date: 2018-12-05 22:49:49 +0900 (週三, 05 十二月 2018) $
+# $Revision: 11295 $ 
 
 import json
 import logging
@@ -35,10 +35,12 @@ from django.views import generic
 from django.utils.translation import ugettext as _
 from django.conf import settings
 from django.http import HttpResponse, Http404
+from django.utils import timezone
 from django.template.loader import get_template
 from django.template import Context
 from paypal.express.views import SuccessResponseView
 from oscar.core.loading import get_model, get_class
+from oscar.core.phonenumber import to_python
 
 from Iuppiter.Encoding import utf8
 from Iuno.shop import forms
@@ -63,6 +65,8 @@ fileHandler = logging.FileHandler('Payment.log')
 fileHandler.setLevel(logging.WARNING)
 logger.addHandler(fileHandler)
 
+DEVELOPMENT = 1
+
 SPGATEWAY = 'spgateway'
 ALLPAY = 'allpay'
 ECPAY = 'ecpay'
@@ -108,6 +112,8 @@ def get_context_data(self, **kwargs):
         kwargs['ecpay'] = buildAPIForm(submission, self.request, 'ecpay')
         kwargs['ecpay_map'] = \
             buildAPIForm(submission, self.request, 'ecpay_map')
+        kwargs['ecpay_logistic'] = \
+            buildAPIForm(submission, self.request, 'ecpay_logistic')
     if allpayEnable:
         kwargs['allpay'] = buildAPIForm(submission, self.request, 'allpay')
     if spgatewayEnable:
@@ -116,7 +122,10 @@ def get_context_data(self, **kwargs):
     if cathaybkEnable:
         kwargs['cathaybk'] = buildAPIForm(
                                         submission, self.request, 'cathaybk')
-    
+
+    if self.request.session.get('logisticMap'):
+        mapData = json.loads(self.request.session['logisticMap'])
+        kwargs.update(**mapData)
     return kwargs
 
 views.PaymentDetailsView.get_context_data = get_context_data
@@ -127,7 +136,7 @@ def download_selected_orders(self, request, orders):
         % self.get_download_filename(request)
     writer = UnicodeCSVWriter(open_file=response)
 
-    meta_data = (('number', _('Order number')),
+    metaData = (('number', _('Order number')),
                     ('value', _('Order value')),
                     ('date', _('Date of purchase')),
                     ('num_items', _('Number of items')),
@@ -140,7 +149,7 @@ def download_selected_orders(self, request, orders):
                     ('product', _('Product')),
                     )
     columns = SortedDict()
-    for k, v in meta_data:
+    for k, v in metaData:
         columns[k] = v
 
     writer.writerow(columns.values())
@@ -189,136 +198,16 @@ def download_selected_orders(self, request, orders):
 
 OrderListView.download_selected_orders = download_selected_orders
 
-from datetime import timedelta
-from decimal import Decimal as D
-from decimal import ROUND_UP
+# https://code.nuwainfo.com/trac/Mercurius/ticket/3155#ticket
+from django.db import models
 
-from oscar.apps.dashboard.views import IndexView
-from oscar.core.compat import get_user_model
-from django.db.models import Avg, Count, Sum
-from django.utils.timezone import now
+class OrderManager(models.Manager):
+    def get_queryset(self):
+        return Order._default_manager.exclude(status=u'測試用訂單')
 
-StockAlert = get_model('partner', 'StockAlert')
-Line = get_model('order', 'Line')
-User = get_user_model()
-
-def get_hourly_report(self, hours=24, segments=10):
-    """
-    Get report of order revenue split up in hourly chunks. A report is
-    generated for the last *hours* (default=24) from the current time.
-    The report provides ``max_revenue`` of the hourly order revenue sum,
-    ``y-range`` as the labeling for the y-axis in a template and
-    ``order_total_hourly``, a list of properties for hourly chunks.
-    *segments* defines the number of labeling segments used for the y-axis
-    when generating the y-axis labels (default=10).
-    """
-    # Get datetime for 24 hours agao
-    time_now = now().replace(minute=0, second=0)
-    start_time = time_now - timedelta(hours=hours - 1)
-
-    # query all OrderNote
-    orders = Order.objects.exclude(status=u'測試用訂單')
-    orders_last_day = orders.filter(date_placed__gt=start_time)
-
-    order_total_hourly = []
-    for hour in range(0, hours, 2):
-        end_time = start_time + timedelta(hours=2)
-        hourly_orders = orders_last_day.filter(date_placed__gt=start_time,
-                                                date_placed__lt=end_time)
-        total = hourly_orders.aggregate(
-            Sum('total_incl_tax')
-        )['total_incl_tax__sum'] or D('0.0')
-        order_total_hourly.append({
-            'end_time': end_time,
-            'total_incl_tax': total
-        })
-        start_time = end_time
-
-    max_value = max([x['total_incl_tax'] for x in order_total_hourly])
-    divisor = 1
-    while divisor < max_value / 50:
-        divisor *= 10
-    max_value = (max_value / divisor).quantize(D('1'), rounding=ROUND_UP)
-    max_value *= divisor
-    if max_value:
-        segment_size = (max_value) / D('100.0')
-        for item in order_total_hourly:
-            item['percentage'] = int(item['total_incl_tax'] / segment_size)
-
-        y_range = []
-        y_axis_steps = max_value / D(str(segments))
-        for idx in reversed(range(segments + 1)):
-            y_range.append(idx * y_axis_steps)
-    else:
-        y_range = []
-        for item in order_total_hourly:
-            item['percentage'] = 0
-
-    ctx = {
-        'order_total_hourly': order_total_hourly,
-        'max_revenue': max_value,
-        'y_range': y_range,
-    }
-    return ctx
-
-def get_stats(self):
-    datetime_24hrs_ago = now() - timedelta(hours=24)
-
-    # query all OrderNote
-    orders = Order.objects.exclude(status=u'測試用訂單')
-    orders_last_day = orders.filter(date_placed__gt=datetime_24hrs_ago)
-
-    open_alerts = StockAlert.objects.filter(status=StockAlert.OPEN)
-    closed_alerts = StockAlert.objects.filter(status=StockAlert.CLOSED)
-
-    total_lines_last_day = Line.objects.filter(
-        order__in=orders_last_day).count()
-    stats = {
-        'total_orders_last_day': orders_last_day.count(),
-        'total_lines_last_day': total_lines_last_day,
-
-        'average_order_costs': orders_last_day.aggregate(
-            Avg('total_incl_tax')
-        )['total_incl_tax__avg'] or D('0.00'),
-
-        'total_revenue_last_day': orders_last_day.aggregate(
-            Sum('total_incl_tax')
-        )['total_incl_tax__sum'] or D('0.00'),
-
-        'hourly_report_dict': self.get_hourly_report(hours=24),
-        'total_customers_last_day': User.objects.filter(
-            date_joined__gt=datetime_24hrs_ago,
-        ).count(),
-
-        'total_open_baskets_last_day': self.get_open_baskets({
-            'date_created__gt': datetime_24hrs_ago
-        }).count(),
-
-        'total_products': Product.objects.count(),
-        'total_open_stock_alerts': open_alerts.count(),
-        'total_closed_stock_alerts': closed_alerts.count(),
-
-        'total_site_offers': self.get_active_site_offers().count(),
-        'total_vouchers': self.get_active_vouchers().count(),
-        'total_promotions': self.get_number_of_promotions(),
-
-        'total_customers': User.objects.count(),
-        'total_open_baskets': self.get_open_baskets().count(),
-        'total_orders': orders.count(),
-        'total_lines': Line.objects.count(),
-        'total_revenue': orders.aggregate(
-            Sum('total_incl_tax')
-        )['total_incl_tax__sum'] or D('0.00'),
-
-        'order_status_breakdown': orders.order_by(
-            'status'
-        ).values('status').annotate(freq=Count('id'))
-    }
-    return stats
-
-IndexView.get_stats = get_stats
-IndexView.get_hourly_report = get_hourly_report
+Order.objects = OrderManager()
 
+# https://code.nuwainfo.com/trac/Mercurius/ticket/3156#ticket
 from django.core.mail import send_mail
 from des.models import DynamicEmailConfiguration
 from django.contrib.sites.models import Site
@@ -333,8 +222,9 @@ def handle_order_status_change(self, order, new_status, note_msg=None):
     viewed as a shipping event affecting all lines.
     """
     order.set_status(new_status)
-    
-    if new_status == u'商品送貨中':
+
+    hasSettings = hasattr(settings, 'OSCAR_ORDER_SEND_EMAIL_STATUS')
+    if hasSettings and new_status == settings.OSCAR_ORDER_SEND_EMAIL_STATUS:
         try:
             temp = get_template(
                 'shop/customer/emails/BeingProcessedSubject.txt')
@@ -473,9 +363,21 @@ class SuccessResponseView2(_PaymentDetailsView):
     
     @method_decorator(csrf_exempt)
     def dispatch(self, request, *args, **kwargs):
+        self.apiName = kwargs['name']
         return super(SuccessResponseView2, self).dispatch(request, 
                                                          *args, **kwargs)
-    
+
+    def get_pre_conditions(self, request):
+        conditions = super(
+            SuccessResponseView2, self).get_pre_conditions(request)
+        if self.apiName == ECPAY_MAP:
+            conditions = [
+                'check_basket_is_not_empty',
+                'check_basket_is_valid',
+                'check_user_email_is_captured',
+            ]
+        return conditions
+
     def post(self, request, *args, **kwargs):
         # apiName
         self.apiName = kwargs['name']
@@ -637,10 +539,6 @@ class SuccessResponseView2(_PaymentDetailsView):
                 self.tradeNo = cathaybkCreditTrade.tradeNo
                 self.tradeAmt = cathaybkCreditTrade.tradeAmt
             elif self.apiName == ECPAY_MAP:
-                submission = super(
-                    SuccessResponseView2, self).build_submission(**kwargs)
-                form = buildAPIForm(submission, self.request, ECPAY_LOGISTIC)
-
                 if hasattr(settings, "ECPAY_API_LOGISTIC_SUB_TYPES"):
                     subTypes = settings.ECPAY_API_LOGISTIC_SUB_TYPES
                 else:
@@ -650,77 +548,161 @@ class SuccessResponseView2(_PaymentDetailsView):
                         "HILIFE": u"萊爾富",
                     }
 
+                extra = request.POST.get("ExtraData")
+                # 防呆 or submission 內會有地址資料
+                splitData = extra.split(",")
+                name = splitData[1][len("name="):]
+                phone = splitData[0][len("phone="):]
+
                 data = {
                     'storeName': self.request.POST.get('CVSStoreName'),
                     'storeAddress': self.request.POST.get('CVSAddress'),
-                    'storeType': subTypes[
+                    'storeTypeName': subTypes[
                         self.request.POST.get('LogisticsSubType')],
+                    'storeId': request.POST.get("CVSStoreID"),
+                    'storeType': self.request.POST.get('LogisticsSubType'),
                     'storePhone': self.request.POST.get('CVSTelephone'),
-                    'form': form,
+                    'receiverName': name,
+                    'receiverPhone': phone,
+                }
+
+                #FIXME: phone 判斷
+                pyPhone = to_python(
+                    data['receiverPhone'].replace('09', '+8869'))
+
+                addressData = {
+                    "first_name": " ", 
+                    "last_name": data['receiverName'],
+                    
+                    "country_id": u'TW', 
+                    "line4": data['storeAddress'], 
+                    "line1": "%s%s" % (
+                        data['storeTypeName'], data['storeName']),
+                    'line3': u'', 
+                    'line2': u'',
+                    "postcode": '00000', 
+                    "phone_number": pyPhone,
+                    'notes': u'',
+                    'state': u'',
+                    'search_text': u'',
+                    'title': u'',  
+                    'id': None,          
                 }
                 
-                return render(request, 'shop/payment/EcpayMapCheck.html', data)
+                self.checkout_session.ship_to_new_address(addressData)
+                
+                self.request.session['logisticMap'] = json.dumps(data)
+               
+                return redirect("checkout:payment-method")
 
             elif self.apiName == ECPAY_LOGISTIC:
                 # data 
-                self.tradeNo = request.POST.get('AllPayLogisticsID')
-                self.tradeAmt = request.POST.get('GoodsAmount')
+                errorMsg = None
+                try:
+                    self.tradeNo = request.POST.get('AllPayLogisticsID')
+                    self.tradeAmt = request.POST.get('GoodsAmount')
 
-                self.merchantTradeNo = request.POST.get('MerchantTradeNo')
-                index = self.merchantTradeNo.find('BASKETID')
-                self.basketId = int(self.merchantTradeNo[index + 8:])
+                    self.merchantTradeNo = request.POST.get('MerchantTradeNo')
+                    index = self.merchantTradeNo.find('BASKETID')
+                    self.basketId = int(self.merchantTradeNo[index + 8:])
 
-                self.merchantID = request.POST.get('MerchantID')
-                self.rtnCode = request.POST.get('RtnCode')
-                self.rtnMsg = request.POST.get('RtnMsg')
-                
-                self.updateStatusDate = request.POST.get('UpdateStatusDate')
-                self.receiverName = request.POST.get('ReceiverName')
-                self.receiverCellPhone = request.POST.get('ReceiverCellPhone')
-                self.receiverEmail = request.POST.get('ReceiverEmail')
-                self.logisticsType = request.POST.get('LogisticsType')
-                self.logisticsSubType = request.POST.get('LogisticsSubType')
+                    self.merchantID = request.POST.get('MerchantID')
+                    self.rtnCode = request.POST.get('RtnCode')
+                    self.rtnMsg = request.POST.get('RtnMsg')
+                    
+                    self.updateStatusDate = request.POST.get('UpdateStatusDate')
+                    self.receiverName = request.POST.get('ReceiverName')
+                    self.receiverCellPhone = request.POST.get(
+                                                'ReceiverCellPhone')
+                    self.receiverEmail = request.POST.get('ReceiverEmail')
+                    self.logisticsType = request.POST.get('LogisticsType')
+                    self.logisticsSubType = request.POST.get('LogisticsSubType')
 
-                self.paymentNo = request.POST.get('CVSPaymentNo')
-                self.validationNo = request.POST.get('CVSValidationNo')
-                
-                # error
-                if self.rtnMsg == 'ERROR':
-                    msg = _(
-                        "There was an error in the payment. Please try again.")
-                    messages.error(self.request, msg)
-                    logger.error(
-                        '%s API ERROR: %s' % (self.apiName, request.POST))
-                    return redirect('checkout:payment-details')
+                    self.paymentNo = request.POST.get('CVSPaymentNo')
+                    self.validationNo = request.POST.get('CVSValidationNo')
+                    
+                    # error
+                    if self.rtnMsg == 'ERROR':
+                        msg = _(
+                            "There was an error in the payment. "
+                            "Please try again.")
+                        messages.error(self.request, msg)
+                        logger.error(
+                            '%s API ERROR: %s' % (self.apiName, request.POST))
+                        return redirect('checkout:payment-details')
 
-                # record
-                """
-                for f in self.request.POST:
-                    print "%s: %s" % (f, self.request.POST[f])
-                """
-                logisticTrade = ECPayLogisticTrade.objects.create(
-                    tradeNo=self.tradeNo,
-                    tradeAmt=int(self.tradeAmt),
-                    merchantId=self.merchantID,
-                    merchantTradeNo=self.merchantTradeNo,
-                    rtnCode=self.rtnCode,
-                    rtnMsg=self.rtnMsg,
-                    tradeDate=self.updateStatusDate.replace('/', '-'),
-                    updateStatusDate=self.updateStatusDate.replace('/', '-'),
-                    receiverName=self.receiverName,
-                    receiverCellPhone=self.receiverCellPhone,
-                    receiverEmail=self.receiverEmail,
-                    logisticsType=self.logisticsType,
-                    logisticsSubType=self.logisticsSubType,
-                    paymentNo=self.paymentNo,
-                    validationNo=self.validationNo,
+                    # record
+                    logisticTrade = ECPayLogisticTrade.objects.create(
+                        tradeNo=self.tradeNo,
+                        tradeAmt=int(self.tradeAmt),
+                        merchantId=self.merchantID,
+                        merchantTradeNo=self.merchantTradeNo,
+                        rtnCode=self.rtnCode,
+                        rtnMsg=self.rtnMsg,
+                        tradeDate=self.updateStatusDate.replace('/', '-'),
+                        updateStatusDate= \
+                            self.updateStatusDate.replace('/', '-'),
+                        receiverName=self.receiverName,
+                        receiverCellPhone=self.receiverCellPhone,
+                        receiverEmail=self.receiverEmail,
+                        logisticsType=self.logisticsType,
+                        logisticsSubType=self.logisticsSubType,
+                        paymentNo=self.paymentNo,
+                        validationNo=self.validationNo,
+                    )
+
+                    if self.request.session.get('paymentData'):
+                        paymentData = json.loads(
+                            self.request.session.get('paymentData'))
+                        self.apiName = paymentData['apiName']
+                        self.tradeNo = "%s|%s" % (
+                            paymentData['tradeNo'], self.tradeNo)
+                except Exception as e:
+                    errorMsg = str(e)
+                    logger.error(errorMsg)
+                finally:
+                    # delete session data
+                    if self.request.session.get('paymentData'):
+                        del self.request.session['paymentData']
+                    if self.request.session.get('logisticMap'):
+                        del self.request.session['logisticMap']
+                    if errorMsg:
+                        msg = _(
+                            "There was an error in the payment. "
+                            "Please try again.")
+                        messages.error(self.request, msg)
+                        return redirect('checkout:payment-details')
+
+
+            if self.request.session.get('logisticMap'):
+                submission = super(
+                    SuccessResponseView2, self).build_submission(**kwargs)
+                form = buildAPIForm(
+                    submission, 
+                    self.request, 
+                    ECPAY_LOGISTIC,
+                    extra={"isCollection": "N",},
                 )
-                
 
+                paymentData = {
+                    'apiName': self.apiName,
+                    'tradeNo': self.tradeNo,
+                }
+                self.request.session['paymentData'] = json.dumps(paymentData)
+
+                ctx = {
+                    'form': form,
+                }
+                return render(
+                    request, 'shop/payment/LogisticRedirect.html', ctx)
             return self.handle_place_order_submission(request)
         except Exception as e:
             # return msg
             logger.error(str(e))
+
+            # raise
+            if settings.SERVER_MODE == DEVELOPMENT:
+                raise
             
             # if payment success
             msg1 = _("There was an except error in the payment."
@@ -825,9 +807,15 @@ class ECPayLogisticreturnView(generic.View):
     def get(self, request, *args, **kwargs):
         orderNumber = kwargs.get('orderNumber')
         order = get_object_or_404(Order, number=orderNumber)
-        source = get_object_or_404(
-            Source, order=order, source_type__name='ecpay_logistic')
-        trade = get_object_or_404(ECPayLogisticTrade, tradeNo=source.reference)
+        source = Source.objects.get(order=order)
+        tradeNo = source.reference.split("|")
+        if len(tradeNo) == 2: # tradeNo|tradeNo
+            tradeNo = tradeNo[1]
+        else:
+            tradeNo = tradeNo[0]
+
+        trade = ECPayLogisticTrade.objects.filter(
+                    tradeNo=tradeNo).order_by('-id').first()
         
         return render(
             request, 
@@ -841,14 +829,16 @@ class ECPayLogisticreturnView(generic.View):
     def post(self, request, *args, **kwargs):
         orderNumber = kwargs.get('orderNumber')
         order = get_object_or_404(Order, number=orderNumber)
-        source = get_object_or_404(
-            Source, order=order, source_type__name='ecpay_logistic')
-        trade = get_object_or_404(ECPayLogisticTrade, tradeNo=source.reference)
-        """
-        if trade.rtnMerchantTradeNo:
-            messages.error(self.request, u"已送出退貨訊息")
-            return redirect('customer:order-list')
-        """
+        source = Source.objects.get(order=order)
+        tradeNo = source.reference.split("|")
+        if len(tradeNo) == 2: # tradeNo|tradeNo
+            tradeNo = tradeNo[1]
+        else:
+            tradeNo = tradeNo[0]
+            
+        trade = ECPayLogisticTrade.objects.filter(
+                    tradeNo=tradeNo).order_by('-id').first()
+       
         data = {
             'MerchantID': trade.merchantId,
             'ServerReplyURL': reverse("ecpayLogisticReply"),
@@ -862,16 +852,22 @@ class ECPayLogisticreturnView(generic.View):
 
         apiUrls = {
             "FAMI": {
-                "stage": "https://logistics-stage.ecpay.com.tw/express/ReturnCVS",
-                "production": "https://logistics-stage.ecpay.com.tw/express/ReturnCVS", 
+                "stage": ("https://logistics-stage.ecpay.com.tw/"
+                         "express/ReturnCVS"),
+                "production": ("https://logistics-stage.ecpay.com.tw/"
+                               "express/ReturnCVS"), 
             },
             "UNIMART": {
-                "stage": "https://logistics-stage.ecpay.com.tw/express/ReturnUniMartCVS",
-                "production": "https://logistics.ecpay.com.tw/express/ReturnUniMartCVS", 
+                "stage": ("https://logistics-stage.ecpay.com.tw/"
+                          "express/ReturnUniMartCVS"),
+                "production": ("https://logistics.ecpay.com.tw/"
+                               "express/ReturnUniMartCVS"), 
             },
             "HILIFE": {
-                "stage": "https://logistics-stage.ecpay.com.tw/express/ReturnHiLifeCVS",
-                "production": "https://logistics.ecpay.com.tw/express/ReturnHiLifeCVS", 
+                "stage": ("https://logistics-stage.ecpay.com.tw/"
+                          "express/ReturnHiLifeCVS"),
+                "production": ("https://logistics.ecpay.com.tw/"
+                               "express/ReturnHiLifeCVS"), 
             },
         }
 
@@ -889,13 +885,28 @@ class ECPayLogisticreturnView(generic.View):
             rtnMerchantTradeNo = res.text.split("|")[0]
             rtnOrderNo = res.text.split("|")[1]
 
-            trade.rtnMerchantTradeNo = rtnMerchantTradeNo 
-            trade.rtnOrderNo = rtnOrderNo
-            trade.rtnMsg = u"已送出退貨資料"
-            trade.rtnCode = 100 # 固定
-            trade.save()
-            
-            order.status = "Order return"
+            newLogisticTrade = ECPayLogisticTrade.objects.create(
+                tradeNo=trade.tradeNo,
+                tradeAmt=int(trade.tradeAmt),
+                merchantId=trade.merchantId,
+                merchantTradeNo=trade.merchantTradeNo,
+                rtnCode=100,
+                rtnMsg=u"已送出退貨資料",
+                tradeDate=trade.tradeDate,
+                updateStatusDate=timezone.now(),
+                receiverName=trade.receiverName,
+                receiverCellPhone=trade.receiverCellPhone,
+                receiverEmail=trade.receiverEmail,
+                logisticsType=trade.logisticsType,
+                logisticsSubType=trade.logisticsSubType,
+                paymentNo=trade.paymentNo,
+                validationNo=trade.validationNo,
+
+                rtnMerchantTradeNo=rtnMerchantTradeNo,
+                rtnOrderNo=rtnOrderNo,
+            )
+
+            order.status = settings.OSCAR_ORDER_RETURN_STATUS
             order.save()
 
             return redirect('customer:order-list')
@@ -937,6 +948,9 @@ class ECPayServerReplyView(generic.View):
 
             self.rtnMerchantTradeNo = request.POST.get('RtnMerchantTradeNo')
 
+            self.paymentNo = request.POST.get('CVSPaymentNo')
+            self.validationNo = request.POST.get('CVSValidationNo')
+
             if self.rtnMerchantTradeNo:
                 logisticTrade = ECPayLogisticTrade.objects.filter(
                     tradeNo=self.tradeNo).update(
@@ -953,28 +967,34 @@ class ECPayServerReplyView(generic.View):
                 order = Order.objects.get(basket=self.basketId)
 
                 if self.rtnCode in ['2030', '3024', '2063', '3018']:
-                    order.status = 'Order being processed'
+                    order.status = settings.OSCAR_ORDER_PROCESSING_STATUS
                 elif self.rtnCode in ['2067', '3022']:
-                    order.status = 'Order complete'
+                    order.status = settings.OSCAR_ORDER_COMPLETE_STATUS
                 elif self.rtnCode in ['2074', '3020']:
-                    order.status = 'Order return'
+                    order.status = settings.OSCAR_ORDER_RETURN_STATUS
                 else:
                     return HttpResponse("|ErrorMessage")
                 order.save()
 
                 logisticTrade = ECPayLogisticTrade.objects.filter(
-                    tradeNo=self.tradeNo).update(
-                        tradeAmt=int(self.tradeAmt),
-                        merchantId=self.merchantID,
-                        merchantTradeNo=self.merchantTradeNo,
-                        rtnCode=self.rtnCode,
-                        rtnMsg=self.rtnMsg,
-                        updateStatusDate=self.updateStatusDate.replace('/', '-'),
-                        receiverName=self.receiverName,
-                        receiverCellPhone=self.receiverCellPhone,
-                        receiverEmail=self.receiverEmail,
-                        logisticsType=self.logisticsType,
-                        logisticsSubType=self.logisticsSubType,
+                    tradeNo=self.tradeNo).order_by('-id').first()
+
+                newLogisticTrade = ECPayLogisticTrade.objects.create(
+                    tradeNo=self.tradeNo,
+                    tradeAmt=int(self.tradeAmt),
+                    merchantId=self.merchantID,
+                    merchantTradeNo=self.merchantTradeNo,
+                    rtnCode=self.rtnCode,
+                    rtnMsg=self.rtnMsg,
+                    tradeDate=logisticTrade.tradeDate,
+                    updateStatusDate=self.updateStatusDate.replace('/', '-'),
+                    receiverName=self.receiverName,
+                    receiverCellPhone=self.receiverCellPhone,
+                    receiverEmail=self.receiverEmail,
+                    logisticsType=self.logisticsType,
+                    logisticsSubType=self.logisticsSubType,
+                    paymentNo=self.paymentNo,
+                    validationNo=self.validationNo,
                 )
 
                 return HttpResponse("1|OK")
